import Player from "../types/player"
import Team from "../types/team"

interface GetParams {
  endpoint: string
  params?: Record<string, string | number | boolean>
}

interface PostParams {
  endpoint: string
  body: unknown
}

export function Get<T>({ endpoint, params }: GetParams): Promise<T> {
  return apiRequest<T>({ method: "GET", endpoint, params })
}

export function Post<T>({ endpoint, body }: PostParams): Promise<T> {
  return apiRequest<T>({ method: "POST", endpoint, body })
}

interface ApiRequestParams {
  method: "GET" | "POST"
  endpoint: string
  body?: unknown
  params?: Record<string, string | number | boolean>
}

async function apiRequest<T>({
  method,
  endpoint,
  body,
  params,
}: ApiRequestParams): Promise<T> {
  const options: RequestInit = {
    method,
  }

  if (method === "POST") {
    options.headers = { "Content-Type": "application/json" }
    if (body) {
      options.body = JSON.stringify(body)
    }
  }

  const queryParams = params
    ? "?" +
      Object.entries(params)
        .map(
          ([key, value]) =>
            encodeURIComponent(key) + "=" + encodeURIComponent(value)
        )
        .join("&")
    : ""

  let resp: Response
  try {
    resp = await fetch(endpoint + queryParams, options)
  } catch (error: unknown) {
    if (error instanceof Error) {
      throw new Error("Network error: " + error.message)
    } else {
      throw new Error("An unknown error occurred")
    }
  }
  if (!resp.ok) {
    const data = await resp.json()
    throw new Error(data.error || "Something went wrong")
  }
  if (resp.status === 204) return null as unknown as T
  return await resp.json()
}

interface SearchMultiResponse {
  players: Player[]
  teams: Team[]
}
export async function SearchMulti(query: string): Promise<SearchMultiResponse> {
  return await Post<SearchMultiResponse>({
    endpoint: "/api/search/multi",
    body: {
      query: query,
      limit: 10,
    },
  })
}
