import { useCallback, useState } from "react"
import { useNavigate } from "react-router-dom"
import AsyncSelect from "react-select/async"
import { SearchMulti } from "../api/api"
import Player from "../types/player"
import Team from "../types/team"

type SearchOption = {
  value: string
  label: string
  imageUrl: string
  type: "player" | "team"
  data: Player | Team
}

type GroupedOptionType = {
  label: string
  options: SearchOption[]
}

export function SearchBox() {
  const [selectedOption, setSelectedOption] = useState<SearchOption | null>(
    null
  )
  const [searchTimeout, setSearchTimeout] = useState<number | null>(null)
  const navigate = useNavigate()

  const handleSearch = useCallback(
    (inputValue: string, callback: (res: GroupedOptionType[]) => void) => {
      if (searchTimeout) {
        clearTimeout(searchTimeout)
      }
      setSearchTimeout(
        setTimeout(() => {
          SearchMulti(inputValue).then((res) => {
            const playerOptions = res.players
              ? res.players.map((player) => ({
                  value: player.slug,
                  label:
                    player.displayName + " (" + player.team?.displayName + ")",
                  type: "player" as const,
                  data: player,
                  imageUrl: player.avatarUrl,
                }))
              : []

            const teamOptions = res.teams
              ? res.teams.map((team) => ({
                  value: team.slug,
                  label: team.displayName,
                  type: "team" as const,
                  data: team,
                  imageUrl: team.pictureUrl,
                }))
              : []

            callback([
              { label: "Players", options: playerOptions },
              { label: "Teams", options: teamOptions },
            ])
          })
        }, 200)
      )
    },
    [searchTimeout]
  )

  const handleSelect = useCallback(
    async (option: SearchOption | null) => {
      if (option) {
        setSelectedOption(null)
        navigate("/" + option.type + "/" + option.value)
      }
    },
    [navigate]
  )

  const formatGroupLabel = (data: GroupedOptionType) => (
    <div>
      <span>{data.label}</span>
    </div>
  )

  const formatOptionLabel = ({ imageUrl, type, data }: SearchOption) => {
    return (
      <div className="flex w-full items-center justify-between">
        <div className="flex items-center gap-2">
          <div className="flex h-10 w-10 items-center justify-center overflow-hidden rounded-full bg-primary-100">
            <img
              className="h-full w-auto max-w-full rounded-full p-0.5"
              src={imageUrl}
              alt=""
            />
          </div>
          <span className="text-md">{data.displayName}</span>
        </div>
        {type === "player" ? (
          <div className="flex items-center justify-end gap-2">
            <span className="text-xs">
              {(data as Player).team?.displayName}
            </span>
            <img
              className="max-w-6"
              src={(data as Player).team?.pictureUrl}
              alt=""
            />
          </div>
        ) : null}
      </div>
    )
  }

  return (
    <AsyncSelect
      value={selectedOption}
      placeholder={"Search..."}
      loadOptions={handleSearch}
      onChange={handleSelect}
      formatGroupLabel={formatGroupLabel}
      formatOptionLabel={formatOptionLabel}
      styles={{
        menu: (base) => ({ ...base, maxHeight: "600px" }),
        menuList: (base) => ({ ...base, maxHeight: "600px" }),
      }}
    />
  )
}
