import { useEffect, useMemo, useState } from "react"
import { GetAvailableFixtures } from "../../api/fixture"
import { GetFixtureGames } from "../../api/game"
import WithDataFetching from "../../components/withdatafetching"
import { Fixture } from "../../types/fixture"
import { Game } from "../../types/game"

export default function Live() {
  return (
    <WithDataFetching<Fixture[]>
      queryKey={["availableFixtures"]}
      queryFn={GetAvailableFixtures}
      refetchInterval={60 * 60000}
    >
      {(data) => <FixtureListLayout fixtures={data} />}
    </WithDataFetching>
  )
}

function FixtureListLayout({ fixtures }: { fixtures: Fixture[] }) {
  const [currentIndex, setCurrentIndex] = useState(() => {
    const startedFixtureIndex = fixtures?.findIndex(
      (fixture) => fixture.fixtureState === "started"
    )
    return startedFixtureIndex !== -1 ? startedFixtureIndex : 0
  })

  useEffect(() => {
    if (currentIndex < 0) {
      setCurrentIndex(0)
    } else if (currentIndex >= fixtures.length) {
      setCurrentIndex(fixtures.length - 1)
    }
  }, [currentIndex, fixtures?.length])

  const selectedFixture = fixtures[currentIndex]

  const handlePrevious = () => {
    if (currentIndex > 0) {
      setCurrentIndex(currentIndex - 1)
    }
  }

  const handleNext = () => {
    if (currentIndex < fixtures.length - 1) {
      setCurrentIndex(currentIndex + 1)
    }
  }
  return (
    <>
      <div className="container mx-auto flex flex-col gap-3 border-x-2 border-neutral-300 bg-neutral-100 px-3 py-6">
        <div className="flex items-center justify-center gap-6">
          {currentIndex < fixtures.length - 1 ? (
            <button onClick={handleNext}>
              <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24">
                <path d="M20 11H7.83l5.59-5.59L12 4l-8 8 8 8 1.41-1.41L7.83 13H20v-2z" />
              </svg>
            </button>
          ) : (
            <div className="w-[24px]"></div>
          )}
          <div className="flex items-center gap-2">
            <span className="text-lg">Gameweek</span>
            <span className="text-lg font-bold">
              {selectedFixture ? selectedFixture.gameWeek : ""}
            </span>
          </div>
          {currentIndex > 0 ? (
            <button onClick={handlePrevious}>
              <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24">
                <path d="M4 11h12.17l-5.59-5.59L12 4l8 8-8 8-1.41-1.41L16.17 13H4v-2z" />
              </svg>
            </button>
          ) : (
            <div className="w-[24px]"></div>
          )}
        </div>
        {selectedFixture && (
          <WithDataFetching<Game[]>
            queryKey={["fixtureGames", selectedFixture.slug]}
            queryFn={() => GetFixtureGames(selectedFixture.slug)}
            refetchInterval={10000}
            enabled={!!selectedFixture}
          >
            {(data) => <FixtureGames fixture={selectedFixture} games={data} />}
          </WithDataFetching>
        )}
      </div>
    </>
  )
}

function areDatesOnSameDay(date1: Date, date2: Date) {
  return (
    date1.getFullYear() === date2.getFullYear() &&
    date1.getMonth() === date2.getMonth() &&
    date1.getDate() === date2.getDate()
  )
}

type TranslationKey = "today" | "tomorrow" | "yesterday"
type LanguageCode =
  | "en"
  | "zh"
  | "hi"
  | "es"
  | "ar"
  | "bn"
  | "pt"
  | "ru"
  | "ja"
  | "pa"
  | "jv"
  | "de"
  | "fr"
  | "it"
  | "tr"
  | "ko"
  | "fa"
  | "uk"
  | "pl"
  | "ro"
  | "nl"
  | "th"
  | "el"
  | "sv"
  | "cs"
type Translations = Record<LanguageCode, Record<TranslationKey, string>>
function getLocalizedWord(word: TranslationKey): string {
  const translations: Translations = {
    en: { today: "Today", tomorrow: "Tomorrow", yesterday: "Yesterday" },
    zh: { today: "今天", tomorrow: "明天", yesterday: "昨天" },
    hi: { today: "आज", tomorrow: "कल", yesterday: "कल" },
    es: { today: "Hoy", tomorrow: "Mañana", yesterday: "Ayer" },
    ar: { today: "اليوم", tomorrow: "غدًا", yesterday: "أمس" },
    bn: { today: "আজ", tomorrow: "আগামীকাল", yesterday: "গতকাল" },
    pt: { today: "Hoje", tomorrow: "Amanhã", yesterday: "Ontem" },
    ru: { today: "Сегодня", tomorrow: "Завтра", yesterday: "Вчера" },
    ja: { today: "今日", tomorrow: "明日", yesterday: "昨日" },
    pa: { today: "ਅੱਜ", tomorrow: "ਕਲ", yesterday: "ਕੱਲ੍ਹ" },
    jv: { today: "Dina iki", tomorrow: "Esuk", yesterday: "Wingi" },
    de: { today: "Heute", tomorrow: "Morgen", yesterday: "Gestern" },
    fr: { today: "Aujourd'hui", tomorrow: "Demain", yesterday: "Hier" },
    it: { today: "Oggi", tomorrow: "Domani", yesterday: "Ieri" },
    tr: { today: "Bugün", tomorrow: "Yarın", yesterday: "Dün" },
    ko: { today: "오늘", tomorrow: "내일", yesterday: "어제" },
    fa: { today: "امروز", tomorrow: "فردا", yesterday: "دیروز" },
    uk: { today: "Сьогодні", tomorrow: "Завтра", yesterday: "Вчора" },
    pl: { today: "Dziś", tomorrow: "Jutro", yesterday: "Wczoraj" },
    ro: { today: "Astăzi", tomorrow: "Mâine", yesterday: "Ieri" },
    nl: { today: "Vandaag", tomorrow: "Morgen", yesterday: "Gisteren" },
    th: { today: "วันนี้", tomorrow: "พรุ่งนี้", yesterday: "เมื่อวาน" },
    el: { today: "Σήμερα", tomorrow: "Αύριο", yesterday: "Χθες" },
    sv: { today: "Idag", tomorrow: "Imorgon", yesterday: "Igår" },
    cs: { today: "Dnes", tomorrow: "Zítra", yesterday: "Včera" },
  }

  const lang = navigator.language.slice(0, 2)
  if ((translations as Record<string, unknown>)[lang]) {
    return translations[lang as LanguageCode][word] || word
  }
  return word
}

function getDayString(day: Date) {
  const today = new Date()
  const todayDate = today.getDate()
  const dayDate = day.getDate()

  if (areDatesOnSameDay(day, today)) {
    return getLocalizedWord("today")
  }
  if (
    dayDate === todayDate + 1 &&
    day.getMonth() === today.getMonth() &&
    day.getFullYear() === today.getFullYear()
  ) {
    return getLocalizedWord("tomorrow")
  }
  if (
    dayDate === todayDate - 1 &&
    day.getMonth() === today.getMonth() &&
    day.getFullYear() === today.getFullYear()
  ) {
    return getLocalizedWord("yesterday")
  }

  const dayName = new Intl.DateTimeFormat(navigator.language, {
    weekday: "long",
  }).format(day)
  return dayName.charAt(0).toUpperCase() + dayName.slice(1)
}

function FixtureGames({ fixture, games }: { fixture: Fixture; games: Game[] }) {
  const [showOnlyStartedAndScheduled, setShowOnlyStartedAndScheduled] =
    useState(false)
  const toggleFilter = () => {
    setShowOnlyStartedAndScheduled((prevState) => !prevState)
  }

  const [selectedDay, setSelectedDay] = useState<Date>(new Date())
  useEffect(() => {
    let initialDate = new Date()
    if (
      initialDate > new Date(fixture.endDate) ||
      initialDate < new Date(fixture.startDate)
    ) {
      initialDate = new Date(fixture.startDate)
    }
    initialDate.setHours(0, 0, 0, 0)
    setSelectedDay(initialDate)
  }, [fixture])

  const uniqueDays = useMemo(() => {
    const daysSet = new Set<string>()

    games?.forEach((game) => {
      const d = new Date(game.date)
      const dateString = `${d.getFullYear()}-${String(
        d.getMonth() + 1
      ).padStart(2, "0")}-${String(d.getDate()).padStart(2, "0")}`
      daysSet.add(dateString)
    })

    return Array.from(daysSet).map((dateString) => new Date(dateString))
  }, [games])

  const filteredGames = useMemo(() => {
    let res = games?.filter((game: Game) =>
      areDatesOnSameDay(new Date(game.date), selectedDay)
    )
    if (showOnlyStartedAndScheduled) {
      res = res?.filter(
        (game: Game) => game.status === "playing" || game.status === "scheduled"
      )
    }
    return res?.sort((a: Game, b: Game) => {
      const time1 = new Date(a.date).getTime()
      const time2 = new Date(b.date).getTime()
      if (time1 === time2) {
        return a.id > b.id ? 1 : -1
      }
      return time1 - time2
    })
  }, [games, showOnlyStartedAndScheduled, selectedDay])

  return (
    <div className="container mx-auto flex flex-col justify-center">
      <div className="flex items-center justify-center gap-4">
        {uniqueDays
          .sort((a: Date, b: Date) => a.getTime() - b.getTime())
          .map((day) => (
            <div className="flex flex-col items-center justify-center gap-1">
              <button
                key={day.getDate()}
                onClick={() => setSelectedDay(day)}
                className={`flex flex-col rounded-lg border bg-primary-100 px-5 py-1 text-neutral-800 shadow-sm ${
                  areDatesOnSameDay(day, selectedDay)
                    ? "bg-primary-200 hover:bg-primary-200"
                    : "hover:border hover:border-primary-300 hover:bg-primary-200"
                } `}
              >
                <span>{getDayString(day)}</span>
                <span className="text-sm">
                  {day.toLocaleDateString(navigator.language, {
                    month: "short",
                    day: "numeric",
                  })}
                </span>
              </button>
            </div>
          ))}
      </div>
      <div className="flex justify-end gap-4 p-4">
        <label
          className="inline-block pl-[0.15rem] hover:cursor-pointer"
          htmlFor="flexSwitchChecked"
        >
          Only started and scheduled
        </label>
        <input
          className="checked:focus:bg-primary dark:checked:bg-primary dark:checked:after:bg-primary mr-2 mt-[0.3rem] h-3.5 w-8 appearance-none rounded-[0.4375rem] bg-neutral-300 before:pointer-events-none before:absolute before:h-3.5 before:w-3.5 before:rounded-full before:bg-transparent before:content-[''] after:absolute after:z-[2] after:-mt-[0.1875rem] after:h-5 after:w-5 after:rounded-full after:border-none after:bg-neutral-100 after:shadow-[0_0px_3px_0_rgb(0_0_0_/_7%),_0_2px_2px_0_rgb(0_0_0_/_4%)] after:transition-[background-color_0.2s,transform_0.2s] after:content-[''] checked:bg-blue-500 checked:after:absolute checked:after:z-[2] checked:after:-mt-[3px] checked:after:ml-[1.0625rem] checked:after:h-5 checked:after:w-5 checked:after:rounded-full checked:after:border-none  checked:after:shadow-[0_3px_1px_-2px_rgba(0,0,0,0.2),_0_2px_2px_0_rgba(0,0,0,0.14),_0_1px_5px_0_rgba(0,0,0,0.12)] checked:after:transition-[background-color_0.2s,transform_0.2s] checked:after:content-[''] hover:cursor-pointer focus:outline-none focus:ring-0 focus:before:scale-100 focus:before:opacity-[0.12] focus:before:shadow-[3px_-1px_0px_13px_rgba(0,0,0,0.6)] focus:before:transition-[box-shadow_0.2s,transform_0.2s] focus:after:absolute focus:after:z-[1] focus:after:block focus:after:h-5 focus:after:w-5 focus:after:rounded-full focus:after:content-[''] checked:focus:border-gray-300 checked:focus:before:ml-[1.0625rem] checked:focus:before:scale-100 checked:focus:before:shadow-[3px_-1px_0px_13px_#3b71ca] checked:focus:before:transition-[box-shadow_0.2s,transform_0.2s] dark:bg-neutral-600 dark:after:bg-neutral-400 dark:focus:before:shadow-[3px_-1px_0px_13px_rgba(255,255,255,0.4)] dark:checked:focus:before:shadow-[3px_-1px_0px_13px_#3b71ca]"
          type="checkbox"
          role="switch"
          id="flexSwitchChecked"
          checked={showOnlyStartedAndScheduled}
          onChange={toggleFilter}
        />
      </div>
      <div className="grid grid-cols-[0.3fr_1fr_150px_1fr_0.3fr] items-center space-y-2">
        {filteredGames.map((game) => (
          <>
            <div className=" px-3 py-1 text-sm">
              {game.status === "playing" ? (
                game.periodType === "HALF_TIME" ? (
                  <span>HT</span>
                ) : (
                  <span>{game.minutes}'</span>
                )
              ) : game.status === "played" ? (
                <span>FT</span>
              ) : game.status === "scheduled" ? (
                <span className={`text-gray-700 `}>
                  {new Date(game.date).toLocaleString(navigator.language, {
                    hour: "numeric",
                    minute: "numeric",
                  })}
                </span>
              ) : null}
            </div>
            <div
              className={`text-md flex items-center justify-end gap-3 ${game.status === "played" && game.winnerTeam?.slug === game.homeTeam.slug ? "font-semibold" : ""}`}
            >
              {game.homeTeam.shortName}
              <div className="flex w-8 justify-center">
                <img className="h-8 " src={game.homeTeam.pictureUrl} alt="" />
              </div>
            </div>
            <div className="flex items-center justify-center px-6">
              <div>
                <div className="flex items-center justify-end gap-1">
                  {game.homePenaltyScore > 0 || game.awayPenaltyScore > 0 ? (
                    <span>({game.homePenaltyScore})</span>
                  ) : null}
                  <span
                    className={`px-2 text-right text-xl ${game.status === "played" && game.winnerTeam?.slug === game.homeTeam.slug ? "font-semibold" : ""}`}
                  >
                    {game.status !== "scheduled" ? game.homeGoals : "-"}
                  </span>
                </div>
              </div>
              {game.status !== "scheduled" && <span>-</span>}
              <div>
                <div className="flex items-center justify-start gap-1">
                  <span
                    className={`px-2 text-right text-xl ${game.status === "played" && game.winnerTeam?.slug === game.awayTeam.slug ? "font-semibold" : ""}`}
                  >
                    {game.status !== "scheduled" ? game.awayGoals : "-"}
                  </span>
                  {game.awayPenaltyScore > 0 || game.homePenaltyScore > 0 ? (
                    <span>({game.awayPenaltyScore})</span>
                  ) : null}
                </div>
              </div>
            </div>
            <div
              className={`px-2 py-1 ${game.status === "played" && game.winnerTeam?.slug === game.awayTeam.slug ? "font-semibold" : ""}`}
            >
              <div className="flex items-center gap-3">
                <div className="flex w-8 justify-center">
                  <img className="h-8 " src={game.awayTeam.pictureUrl} alt="" />
                </div>
                {game.awayTeam.shortName}
              </div>
            </div>

            <div className="flex items-center justify-end gap-3">
              <div className="text-right text-2xs">
                {game.competition.displayName}
              </div>
              <div>
                <img
                  className="h-4 w-6 border border-neutral-300 shadow-sm"
                  src={game.competition.country?.flagFlat32Url}
                  alt={`${game.competition.displayName} (${game.competition.country?.displayName})`}
                />
              </div>
            </div>
          </>
        ))}
      </div>
    </div>
  )
}
