import {
  addMonths,
  format,
  isSameDay,
  startOfMonth,
  startOfYear,
  subMonths,
} from "date-fns"
import moment from "moment/moment"
import { useMemo } from "react"
import {
  Cell,
  ComposedChart,
  LabelList,
  ReferenceLine,
  ResponsiveContainer,
  Scatter,
  Tooltip,
  XAxis,
  YAxis,
} from "recharts"
import {
  Game,
  GamePlayer,
  GamePlayerScore,
  GetScoreColor,
} from "../../types/game"

type DataPoint = {
  date: Date
  matchScore?: number
  decisiveScore?: number
  aaScore?: number
  game?: Game
  score?: GamePlayerScore
  gameweek: number
  dateAsNumber: number
}

const CustomTooltip = ({ active, payload }: any) => {
  if (active && payload && payload.length) {
    const dataPoint = payload[0].payload

    const { gameweek, date } = dataPoint
    const formattedDate = moment(date).format("DD MMM YYYY")

    return (
      <div className="rounded-md border border-gray-300 bg-white p-2">
        <p className="text-black">{`Date: ${formattedDate}`}</p>
        {gameweek && <p className="text-black">{`Gameweek: ${gameweek}`}</p>}
        {payload.map((item: any) => (
          <p
            key={item.name}
            style={{ color: item.color }}
          >{`${item.name}: ${Math.round(item.value)}`}</p>
        ))}
      </div>
    )
  }

  return null
}

export default function ScoreGraph({
  scores,
  startDate,
  endDate,
}: {
  scores: GamePlayer[]
  startDate: Date
  endDate: Date
}) {
  const CustomXAxisTick = ({ x, y, payload }: any) => {
    const date = new Date(payload.value)
    const isFirstJan = isSameDay(date, startOfYear(date))
    const isFirstOfMonth = isSameDay(date, startOfMonth(date))
    const displayText = isFirstJan
      ? format(date, "yyyy")
      : isFirstOfMonth
        ? format(date, "MMM")
        : ""

    return (
      <g transform={`translate(${x},${y})`}>
        <text x={0} y={0} dy={16} textAnchor="middle" fill="#666">
          {displayText}
        </text>
      </g>
    )
  }

  const adjustedStartDate = useMemo(() => {
    const earliestScoreDate = scores.reduce((minDate, score) => {
      const scoreDate = new Date(score.game?.date ?? "")
      return minDate > scoreDate ? scoreDate : minDate
    }, new Date())
    return startDate > earliestScoreDate ? startDate : earliestScoreDate
  }, [scores, startDate])

  const filteredScores = useMemo(
    () =>
      scores
        .filter(
          (score) =>
            new Date(score.game?.date ?? "") >= startDate &&
            new Date(score.game?.date ?? "") <= endDate
        )
        .map<DataPoint>((score) => ({
          date: score.game?.fixture.startDate ?? new Date(),
          matchScore: score.score?.score,
          decisiveScore: score.score?.decisiveScore,
          aaScore: score.score?.allAroundScore,
          game: score.game,
          score: score.score,
          gameweek: score.game?.fixture.gameWeek ?? 0,
          dateAsNumber:
            new Date(score.game?.fixture.startDate ?? new Date()).getTime() ??
            0,
        }))
        .sort((a, b) => a.gameweek - b.gameweek),
    [scores, startDate, endDate]
  )

  const ticks = useMemo(() => {
    const start = startOfMonth(adjustedStartDate)
    const end = startOfMonth(endDate)
    let current = start
    const dates = []
    while (current <= end) {
      dates.push(current.getTime())
      current = addMonths(current, 1)
    }
    dates.unshift(subMonths(start, 1).getTime())
    dates.push(addMonths(end, 1).getTime())
    return dates
  }, [adjustedStartDate, endDate])

  return (
    <div className="z-10 flex w-full flex-col gap-6">
      <ResponsiveContainer width="100%" height={500}>
        <ComposedChart data={filteredScores}>
          <XAxis
            dataKey="dateAsNumber"
            domain={["dataMin", "dataMax"]}
            name="Date"
            type="number"
            tick={<CustomXAxisTick />}
            ticks={ticks}
          />
          <YAxis domain={[-10, 110]} ticks={[0, 20, 40, 60, 80, 100]} />
          <Scatter dataKey="matchScore" name="Score" isAnimationActive={false}>
            {filteredScores?.map((entry, index) => (
              <Cell
                key={`cell-${index}`}
                fill={GetScoreColor(entry.matchScore ?? 0, false)}
              />
            ))}
            <LabelList
              dataKey="matchScore"
              position="top"
              offset={10}
              stroke={"#333"}
              strokeWidth={0.3}
              fontSize={12}
              formatter={(value: number) => Math.round(value)}
            />
          </Scatter>
          {[0, 20, 40, 60, 80, 100].map((value) => (
            <ReferenceLine
              key={value}
              y={value}
              stroke="gray"
              strokeWidth={0.3}
            />
          ))}

          <Tooltip content={<CustomTooltip />} />
        </ComposedChart>
      </ResponsiveContainer>
    </div>
  )
}
