import moment from "moment"
import React from "react"
import { BiFootball } from "react-icons/bi"
import { BsHandThumbsUp } from "react-icons/bs"
import { FaHandSparkles } from "react-icons/fa"
import { GiFist, GiFootprint, GiLeg, GiWhistle } from "react-icons/gi"
import { TbHandStop } from "react-icons/tb"
import {
  GamePlayer,
  GetAAScoreColor,
  GetDSScoreColor,
  GetMinutesScoreColor,
  GetScoreColor,
} from "../../types/game"
import Player from "../../types/player"
import CardIcon from "./components/card_icon"
import { FilterSelect } from "./components/filter_select"
import ScoreBox from "./components/scorebox"

type GameEventProps = {
  count: number | undefined
  icon: React.ElementType
  positive?: boolean
  label?: string
}

function GameEvent({
  count,
  icon: Icon,
  positive = true,
  label = "",
}: GameEventProps) {
  if (!count || count === 0) {
    return null
  }

  return (
    <div title={label} className="flex flex-col items-center">
      <span className="text-xs font-semibold">{count}</span>
      <div className={`${positive ? "text-green-700" : "text-red-600"}`}>
        <Icon />
      </div>
    </div>
  )
}

function ScoreText({ score }: { score: number | string }) {
  return (
    <td className="text-center">
      <span className="text-xs font-bold text-gray-600">{score}</span>
    </td>
  )
}

function ScoreHeader({ label, tooltip }: { label: string; tooltip: string }) {
  return (
    <th title={tooltip} className="text-center">
      {label}
    </th>
  )
}

export default function ScoreTable({
  player,
  scores,
}: {
  player: Player
  scores: GamePlayer[]
}) {
  const [statCategory, setStatCategory] = React.useState("def")

  let availableStatCategory = [
    { value: "def", label: "Defensive" },
    { value: "off", label: "Offensive" },
    { value: "poss", label: "Possession" },
  ]
  if (player.fieldPosition === "Goalkeeper") {
    availableStatCategory = [
      ...availableStatCategory,
      { value: "goal", label: "Goalkeeping" },
    ]
  }

  return (
    <div className="flex w-full flex-col gap-6">
      <div className="flex w-full justify-end">
        <FilterSelect
          value={statCategory}
          options={availableStatCategory}
          onChange={setStatCategory}
        />
      </div>
      <table>
        <thead className="bg-neutral-50">
          <tr>
            <th title="Game Week" className="th">
              GW
            </th>
            <th>Date</th>
            <th title="Minutes played" className="th text-center">
              Minutes
            </th>
            <th className="text-right">Home</th>
            <th></th>
            <th></th>
            <th></th>
            <th>Away</th>
            <th className="text-center">Score</th>
            <th></th>
            <th title="Decisive score" className="text-center">
              DS
            </th>
            <th title="All-around score" className="text-center">
              AA
            </th>

            {statCategory === "def" && (
              <>
                <ScoreHeader tooltip={"Clean sheet"} label={"CS"} />
                <ScoreHeader tooltip={"Error lead to goal"} label={"ELG"} />
                <ScoreHeader tooltip={"Goals conceded"} label={"GC"} />
                <ScoreHeader tooltip={"Effective clearance"} label={"EC"} />
                <ScoreHeader tooltip={"Won tackle"} label={"WT"} />
                <ScoreHeader tooltip={"Blocked cross"} label={"BC"} />
                <ScoreHeader tooltip={"Block"} label={"B"} />
              </>
            )}
            {statCategory === "off" && (
              <>
                <ScoreHeader tooltip={"Shots on target"} label={"SOT"} />
                <ScoreHeader tooltip={"Won contest"} label={"WC"} />
                <ScoreHeader tooltip={"Big chance created"} label={"BCC"} />
                <ScoreHeader tooltip={"Attempted assist"} label={"AA"} />
                <ScoreHeader tooltip={"Penalty area entries"} label={"PAE"} />
                <ScoreHeader tooltip={"Penalty kick missed"} label={"PKM"} />
                <ScoreHeader tooltip={"Big chance missed"} label={"BCM"} />
              </>
            )}
            {statCategory === "poss" && (
              <>
                <ScoreHeader tooltip={"Possession +won -lost"} label={"Poss"} />
                <ScoreHeader tooltip={"Duel +won -lost"} label={"Duel"} />
                <ScoreHeader tooltip={"Interceptions"} label={"Int"} />
                <ScoreHeader tooltip={"Accurate pass"} label={"AP"} />
                <ScoreHeader
                  tooltip={"Accurate final 3rd pass"}
                  label={"A3P"}
                />
                <ScoreHeader tooltip={"Accurate long balls"} label={"ALB"} />
                <ScoreHeader
                  tooltip={"Long pass into opposition"}
                  label={"LPO"}
                />
                <ScoreHeader tooltip={"Missed pass"} label={"MP"} />
                <ScoreHeader tooltip={"Pass accuracy"} label={"PA%"} />
              </>
            )}
            {statCategory === "goal" && (
              <>
                <ScoreHeader tooltip={"Saves"} label={"S"} />
                <ScoreHeader
                  tooltip={"Saved shot from inside the box"}
                  label={"SIB"}
                />
                <ScoreHeader tooltip={"Good high claim"} label={"GHC"} />
                <ScoreHeader tooltip={"Punches"} label={"P"} />
                <ScoreHeader tooltip={"Diving save"} label={"DS"} />
                <ScoreHeader tooltip={"Diving catch"} label={"DC"} />
                <ScoreHeader tooltip={"Cross not claimed"} label={"CNC"} />
                <ScoreHeader tooltip={"GK smother"} label={"GKS"} />
                <ScoreHeader tooltip={"Keeper sweeper"} label={"KS"} />
              </>
            )}
          </tr>
        </thead>
        <tbody>
          {scores
            ?.sort((a, b) =>
              new Date(a.game?.date ?? new Date()) <
              new Date(b.game?.date ?? new Date())
                ? 1
                : -1
            )
            .map((scoreData) => {
              if (!scoreData.game || !scoreData.score) return null
              return (
                <tr key={scoreData.score.gameId}>
                  <td>{scoreData.game.fixture?.gameWeek}</td>
                  <td>{moment(scoreData.game.date).format("DD MMMM YYYY")}</td>
                  <td className="text-center">
                    <span
                      className={`w-10 rounded-md border-2 border-gray-300 p-1 text-white ${GetMinutesScoreColor(scoreData.score?.minutesPlayed ?? 0)}`}
                    >
                      {scoreData.score.minutesPlayed}
                    </span>
                  </td>
                  <td className="text-right">
                    <span className="text-xs">
                      {scoreData.game.homeTeam.displayName}
                    </span>
                  </td>
                  <td className="text-right">{scoreData.game.homeGoals}</td>
                  <td className="text-center">-</td>
                  <td>{scoreData.game.awayGoals}</td>
                  <td>
                    <span className="text-xs">
                      {scoreData.game.awayTeam.displayName}
                    </span>
                  </td>
                  <td className="text-center">
                    <ScoreBox
                      score={scoreData.score.score}
                      colorfunc={GetScoreColor}
                    />
                  </td>
                  <td>
                    <div className="flex justify-end">
                      <GameEvent
                        count={scoreData.score.goal}
                        icon={BiFootball}
                        label="Goal"
                      />
                      <GameEvent
                        count={scoreData.score.assist}
                        icon={GiFootprint}
                        label="Assist"
                      />
                      <GameEvent
                        count={scoreData.score.penaltyWon}
                        icon={GiWhistle}
                        label="Penalty won"
                      />
                      <GameEvent
                        count={scoreData.score.clearanceOffLine}
                        icon={TbHandStop}
                        label="Clearance off line"
                      />
                      <GameEvent
                        count={
                          player.fieldPosition === "Goalkeeper" &&
                          scoreData.score.cleanSheet
                            ? 1
                            : 0
                        }
                        icon={FaHandSparkles}
                        label="Clean sheet"
                      />
                      <GameEvent
                        count={scoreData.score.penaltySave}
                        icon={GiFist}
                        label="Penalty saved"
                      />
                      <GameEvent
                        count={scoreData.score.lastManTackle}
                        icon={GiLeg}
                        label="Last man tackle"
                      />
                      <GameEvent
                        count={scoreData.score.redCard ? 1 : 0}
                        icon={() => <CardIcon color={"#FF0000"} />}
                        label="Red card"
                      />
                      <GameEvent
                        count={scoreData.score.yellowCard ? 1 : 0}
                        icon={() => <CardIcon color={"#FFD700"} />}
                        label="Yellow card"
                      />
                    </div>
                  </td>

                  <td className="text-center">
                    <ScoreBox
                      score={scoreData.score.decisiveScore}
                      colorfunc={GetDSScoreColor}
                    />
                  </td>
                  <td className="text-center">
                    <ScoreBox
                      score={scoreData.score?.allAroundScore ?? 0}
                      colorfunc={GetAAScoreColor}
                    />
                  </td>
                  {statCategory === "def" &&
                    (scoreData.score.minutesPlayed ?? 0) > 0 && (
                      <>
                        <td className="flex justify-center">
                          {scoreData.score.cleanSheet ? <BsHandThumbsUp /> : ""}
                        </td>
                        <ScoreText score={scoreData.score.errorLeadToGoal} />
                        <ScoreText score={scoreData.score.goalsConceded} />
                        <ScoreText score={scoreData.score.effectiveClearance} />
                        <ScoreText score={scoreData.score.wonTackle} />
                        <ScoreText score={scoreData.score.blockedCross} />
                        <ScoreText score={scoreData.score.block} />
                      </>
                    )}
                  {statCategory === "off" &&
                    scoreData.score.minutesPlayed > 0 && (
                      <>
                        <ScoreText score={scoreData.score.shotOnTarget} />
                        <ScoreText score={scoreData.score.wonContest} />
                        <ScoreText score={scoreData.score.bigChanceCreated} />
                        <ScoreText score={scoreData.score.attemptedAssist} />
                        <ScoreText score={scoreData.score.penaltyAreaEntries} />
                        <ScoreText score={scoreData.score.penaltyKickMissed} />
                        <ScoreText score={scoreData.score.bigChanceMissed} />
                      </>
                    )}
                  {statCategory === "poss" &&
                    scoreData.score.minutesPlayed > 0 && (
                      <>
                        <ScoreText
                          score={`+${scoreData.score.possessionWon} -${scoreData.score.possessionLost}`}
                        />
                        <ScoreText
                          score={`+${scoreData.score.duelWon} -${scoreData.score.duelLost}`}
                        />
                        <ScoreText score={scoreData.score.interception} />
                        <ScoreText score={scoreData.score.accuratePass} />
                        <ScoreText
                          score={scoreData.score.accurateFinalThirdPass}
                        />
                        <ScoreText score={scoreData.score.accurateLongBall} />
                        <ScoreText
                          score={scoreData.score.longPassIntoOpposition}
                        />
                        <ScoreText score={scoreData.score.missedPass} />
                        <ScoreText
                          score={
                            Number(
                              (scoreData.score.accuratePass /
                                (scoreData.score.accuratePass +
                                  scoreData.score.missedPass)) *
                                100
                            ).toFixed(1) + "%"
                          }
                        />
                      </>
                    )}
                  {statCategory === "goal" &&
                    scoreData.score.minutesPlayed > 0 && (
                      <>
                        <ScoreText score={scoreData.score.saves} />
                        <ScoreText
                          score={scoreData.score.savedShotFromInsideBox}
                        />
                        <ScoreText score={scoreData.score.goodHighClaim} />
                        <ScoreText score={scoreData.score.punches} />
                        <ScoreText score={scoreData.score.divingSave} />
                        <ScoreText score={scoreData.score.divingCatch} />
                        <ScoreText score={scoreData.score.crossNotClaimed} />
                        <ScoreText score={scoreData.score.goalkeeperSmother} />
                        <ScoreText score={scoreData.score.keeperSweeper} />
                      </>
                    )}
                </tr>
              )
            })}
        </tbody>
      </table>
    </div>
  )
}
