import { useMemo } from "react"
import { SingleRankingsParams } from "../../api/rankings"
import InputWrapper from "../../components/input_wrapper"
import Competition from "../../types/competition"

interface ZoneGroup {
  label: string
  options: Array<{ label: string; value: string }>
}

interface CompetitionsSelectProps {
  competitions: Competition[]
  params: SingleRankingsParams
  setParams: (params: SingleRankingsParams) => void
}

function CompetitionsSelect({
  competitions,
  params,
  setParams,
}: CompetitionsSelectProps) {
  const zoneGroups = useMemo(() => {
    const groups: Record<string, ZoneGroup> = {}
    competitions.forEach((competition) => {
      if (!competition.zone) return
      if (!groups[competition.zone.id]) {
        groups[competition.zone.id] = {
          label: competition.zone?.displayName,
          options: [],
        }
      }
      groups[competition.zone.id].options.push({
        label: competition.displayName,
        value: competition.slug,
      })
    })
    return groups
  }, [competitions])

  const groupedOptions = useMemo(() => {
    const sortedGroups = Object.values(zoneGroups).sort((a, b) => {
      if (a.label === "") return 1
      if (b.label === "") return -1
      return a.label.localeCompare(b.label)
    })

    const withoutZoneGroup = {
      label: "Without zone",
      options: sortedGroups.find((group) => group.label === "")?.options || [],
    }

    const filteredGroups = sortedGroups
      .filter((group) => group.label !== "")
      .map((group) => ({
        ...group,
        options: group.options.sort((a, b) => a.label.localeCompare(b.label)),
      }))

    if (withoutZoneGroup.options.length > 0) {
      filteredGroups.push(withoutZoneGroup)
    }

    return filteredGroups
  }, [zoneGroups])

  const selectedOptions = useMemo(() => {
    return params.competitions.map((slug) => {
      const competition = competitions.find((c) => c.slug === slug)
      return { value: slug, label: competition?.displayName || "" }
    })
  }, [competitions, params.competitions])

  return (
    <InputWrapper label={"Competitions"} id={"competitions"}>
      <select
        // multiple={true}
        value={selectedOptions.map((option) => option.value)}
        onChange={(e) => {
          const selectedValues = Array.from(
            e.target.selectedOptions,
            (option) => option.value
          )
          const nonEmptySelectedValues = selectedValues.filter(
            (value) => value !== ""
          )
          setParams({
            ...params,
            competitions:
              nonEmptySelectedValues.length > 0 ? nonEmptySelectedValues : [],
          })
        }}
        className="w-full rounded-lg border border-neutral-300 bg-neutral-50 py-2 pl-3 pr-2 text-sm text-neutral-900 shadow focus:border-primary-blue-300 focus:ring-primary-blue-300"
      >
        <option value="">No Selection</option>
        {groupedOptions.map((group, groupIndex) => (
          <optgroup key={groupIndex} label={group.label}>
            {group.options.map((option, optionIndex) => (
              <option key={optionIndex} value={option.value}>
                {option.label}
              </option>
            ))}
          </optgroup>
        ))}
      </select>

      {/* <Select
        value={selectedOptions}
        placeholder="Select competitions..."
        options={groupedOptions}
        isMulti
        formatGroupLabel={(group) => (
          <div className="bg-primary-blue-500 text-white font-bold px-2 py-1 rounded-md text-center border border-neutral-200 shadow-sm mb-3">
            {group.label}
          </div>
        )}
        formatOptionLabel={(option) => (
          <div className="px-1 text-md">{option.label}</div>
        )}
        onChange={(newValue) => {
          setParams({
            ...params,
            competitions: newValue.map((option) => option.value),
          });
        }}
        styles={{
          control: (base) => ({
            ...base,
            borderRadius: "0.5rem",
          }),
        }}
      /> */}
    </InputWrapper>
  )
}

export default CompetitionsSelect
