import { useQuery } from "@tanstack/react-query"
import { ChangeEvent, useEffect, useState } from "react"
import { Link } from "react-router-dom"
import { GetClubCompetitions } from "../../api/competition"
import { GetSingleRankings, SingleRankingsParams } from "../../api/rankings"
import Checkbox from "../../components/checkbox"
import InputWrapper from "../../components/input_wrapper"
import Loader from "../../components/loader"
import NumericInput from "../../components/numeric_input"
import CompetitionsSelect from "./competition_select"
import ProgressBar from "./components/progress_bar"
import PositionSelect from "./position_select"
import ZoneSelect from "./zone_select"

function calculateAge(birthDate: Date): number {
  const today = new Date()
  let age = today.getFullYear() - birthDate.getFullYear()
  const m = today.getMonth() - birthDate.getMonth()

  if (m < 0 || (m === 0 && today.getDate() < birthDate.getDate())) {
    age--
  }

  return age
}

// const rarities = [
//   { value: "limited", label: "Limited" },
//   { value: "rare", label: "Rare" },
//   { value: "super_rare", label: "Super Rare" },
//   { value: "unique", label: "Unique" },
// ];

const orders = [
  {
    value: "total_avg_score DESC",
    label: "Score (High to Low)",
  },
  {
    value: "decisive_avg_score DESC",
    label: "Decisive (High to Low)",
  },
  {
    value: "all_around_avg_score DESC",
    label: "All Around (High to Low)",
  },
  {
    value: "aa_per_min DESC",
    label: "AA/Min (High to Low)",
  },
  {
    value: "more_than_80_score_percentage DESC",
    label: "Score Percentage > 80 (High to Low)",
  },
  {
    value: "more_than_70_score_percentage DESC",
    label: "Score Percentage > 70 (High to Low)",
  },
  {
    value: "more_than_60_score_percentage DESC",
    label: "Score Percentage > 60 (High to Low)",
  },
  {
    value: "more_than_50_score_percentage DESC",
    label: "Score Percentage > 50 (High to Low)",
  },
  {
    value: "more_than_40_aa_percentage DESC",
    label: "AA Percentage > 40 (High to Low)",
  },
  {
    value: "more_than_30_aa_percentage DESC",
    label: "AA Percentage > 30 (High to Low)",
  },
  {
    value: "more_than_20_aa_percentage DESC",
    label: "AA Percentage > 20 (High to Low)",
  },
  {
    value: "more_than_10_aa_percentage DESC",
    label: "AA Percentage > 10 (High to Low)",
  },
  {
    value: "duel_won_per_90_min DESC",
    label: "Duel Won Per 90 Min (High to Low)",
  },
  {
    value: "effective_clearance_per_90_min DESC",
    label: "Effective Clearance Per 90 Min (High to Low)",
  },
  {
    value: "shot_on_target_per_90_min DESC",
    label: "Shot On Target Per 90 Min (High to Low)",
  },
  {
    value: "accurate_pass_per_90_min DESC",
    label: "Accurate Pass Per 90 Min (High to Low)",
  },
  {
    value: "won_contest_per_90_min DESC",
    label: "Won Contest Per 90 Min (High to Low)",
  },
]

function SingleRankingsPage() {
  const [params, setParams] = useState<SingleRankingsParams>({
    limit: 100,
    startDate: new Date(new Date().setMonth(new Date().getMonth() - 3)),
    competitions: [],
    zones: [],
    minGameCount: 5,
    minTeamGamesPlayedPercentage: 70,
    onlyClubGames: true,
    onlyStarting: true,
    order: "total_avg_score DESC",
    rarity: "limited",
    position: "Defender",
    u23: false,
    minTotalMinutes: 0,
    minAge: 0,
    maxAge: 40,
    hasGameInNextGw: false,
  })

  const [usedParams, setUsedParams] = useState<SingleRankingsParams>(params)

  const {
    data,
    isFetching,
    refetch: refetchRankings,
  } = useQuery({
    queryKey: ["singleRankings"],
    queryFn: async () => {
      const rankings = await GetSingleRankings(params)
      setUsedParams(params)
      return rankings
    },
    refetchOnWindowFocus: false,
    refetchOnMount: false,
    refetchOnReconnect: false,
  })

  const { data: competitions, refetch: refetchCompetitions } = useQuery({
    queryKey: ["competitions", params.zones],
    queryFn: async () => {
      const competitions = await GetClubCompetitions(params.zones)
      return competitions.sort((a, b) =>
        a.displayName.localeCompare(b.displayName)
      )
    },
  })

  useEffect(() => {
    setParams((prevParams) => ({
      ...prevParams,
      competitions: [],
    }))

    refetchCompetitions()
  }, [params.zones, refetchCompetitions])

  return (
    <>
      <div className="flex items-center justify-between rounded border-b border-neutral-300 bg-primary-100 px-3 py-1 shadow-lg">
        <div className="flex flex-col">
          <div className="flex w-full items-center justify-center gap-3 p-3">
            <PositionSelect params={params} setParams={setParams} />
            <ZoneSelect params={params} setParams={setParams}></ZoneSelect>
          </div>
          <div className={"grid grid-cols-[1fr_auto_auto]"}>
            <div className="flex w-full flex-wrap items-baseline justify-around gap-6 px-8 py-3">
              <InputWrapper label={"Start Date"} id={"startDate"}>
                <input
                  defaultValue={
                    params.startDate
                      ? params.startDate.toISOString().slice(0, 10)
                      : ""
                  }
                  className="w-40 rounded-lg border border-neutral-300 bg-neutral-50 py-2 pl-3 pr-2 text-sm text-neutral-900 shadow focus:border-primary-blue-300 focus:ring-primary-blue-300"
                  id="startDate"
                  type="date"
                  placeholder="Start Date"
                  onChange={(e) => {
                    if (e.target.value) {
                      setParams({
                        ...params,
                        startDate: new Date(e.target.value),
                      })
                    }
                  }}
                />
              </InputWrapper>

              <NumericInput
                value={params.minGameCount}
                label={"Min Game Count"}
                id={"minGameCount"}
                onChange={(value) => {
                  setParams({
                    ...params,
                    minGameCount: value,
                  })
                }}
              />

              <NumericInput
                value={params.minTeamGamesPlayedPercentage}
                label={"Min Team Games Played Percentage"}
                id={"minTeamGamesPlayedPercentage"}
                onChange={(value) => {
                  setParams({
                    ...params,
                    minTeamGamesPlayedPercentage: value,
                  })
                }}
              />

              <div className="w-[250px]">
                <CompetitionsSelect
                  competitions={competitions ?? []}
                  params={params}
                  setParams={setParams}
                />
              </div>

              <NumericInput
                value={params.minAge}
                label={"Min Age"}
                id={"minAge"}
                onChange={(value) => {
                  setParams({
                    ...params,
                    minAge: value,
                  })
                }}
              />
              <NumericInput
                value={params.maxAge}
                label={"Max Age"}
                id={"maxAge"}
                onChange={(value) => {
                  setParams({
                    ...params,
                    maxAge: value,
                  })
                }}
              />

              <NumericInput
                value={params.minTotalMinutes}
                label={"Min Total Minutes"}
                id={"minTotalMinutes"}
                onChange={(value) => {
                  setParams({
                    ...params,
                    minTotalMinutes: value,
                  })
                }}
              />
            </div>
          </div>
        </div>
        <div className={"flex flex-col gap-2 p-3"}>
          <Checkbox
            id="onlyClubGamesCheckbox"
            checked={params.onlyClubGames}
            label={"Only club games"}
            onChange={(e: ChangeEvent<HTMLInputElement>) => {
              setParams({
                ...params,
                onlyClubGames: e.target.checked,
              })
            }}
          />
          <Checkbox
            id="onlyStartingGamesCheckbox"
            checked={params.onlyStarting}
            label={"Only starting games"}
            onChange={(e: ChangeEvent<HTMLInputElement>) => {
              setParams({
                ...params,
                onlyStarting: e.target.checked,
              })
            }}
          />
          <Checkbox
            id="u23Checkbox"
            checked={params.u23}
            label={"U23"}
            onChange={(e: ChangeEvent<HTMLInputElement>) => {
              setParams({
                ...params,
                u23: e.target.checked,
              })
            }}
          />
          <Checkbox
            id="hasGameInNextGw"
            checked={params.hasGameInNextGw}
            label={"Has game in next GW"}
            onChange={(e: ChangeEvent<HTMLInputElement>) => {
              setParams({
                ...params,
                hasGameInNextGw: e.target.checked,
              })
            }}
          />
        </div>
        <div className={"flex flex-col items-center justify-center gap-3 p-3"}>
          <NumericInput
            value={params.limit}
            label={"Limit"}
            id={"limit"}
            onChange={(value) => {
              setParams({
                ...params,
                limit: value,
              })
            }}
          />
          <InputWrapper label={"Order by"} id={"orders"}>
            <select
              defaultValue={params.order}
              onChange={(e) => {
                setParams({
                  ...params,
                  order: e.target.value,
                })
              }}
              className="w-40 rounded-lg border border-neutral-300 bg-neutral-50 py-2 pl-3 pr-2 text-sm text-neutral-900 shadow focus:border-primary-blue-300 focus:ring-primary-blue-300"
            >
              {orders.map(({ label, value }) => (
                <option key={value} value={value}>
                  {label}
                </option>
              ))}
            </select>
          </InputWrapper>
          <button
            className={`btn flex w-32 items-center justify-center gap-3 ${
              isFetching
                ? "bg-primary-blue-300"
                : "bg-primary-blue-500 hover:bg-primary-blue-700"
            } font-bold`}
            type="button"
            onClick={() => refetchRankings()}
            disabled={isFetching}
          >
            {isFetching ? (
              <>
                <>
                  <div
                    className="h-4 w-4 animate-spin rounded-full border-[1px] border-current border-t-transparent text-blue-600"
                    role="status"
                    aria-label="loading"
                  ></div>
                  <span>Loading...</span>
                </>
              </>
            ) : (
              <span>Apply Filters</span>
            )}
          </button>
        </div>
      </div>
      {isFetching ? (
        <Loader></Loader>
      ) : (
        <div className="overflow-x-auto p-3">
          <table className="divide-y divide-neutral-200">
            <thead className="bg-neutral-50">
              <tr>
                <th></th>
                <th className="px-6 py-3 text-center text-xs font-medium uppercase tracking-wider text-neutral-500">
                  Team
                </th>
                <th className="px-6 py-3 text-center text-xs font-medium uppercase tracking-wider text-neutral-500">
                  Name
                </th>
                <th className="px-6 py-3 text-center text-xs font-medium uppercase tracking-wider text-neutral-500">
                  Age
                </th>
                <th className="px-6 py-3 text-center text-xs font-medium uppercase tracking-wider text-neutral-500">
                  Score
                </th>
                <th className="px-6 py-3 text-center text-xs font-medium uppercase tracking-wider text-neutral-500">
                  Stddev
                </th>
                <th className="px-6 py-3 text-center text-xs font-medium uppercase tracking-wider text-neutral-500">
                  DS
                </th>
                <th className="px-6 py-3 text-center text-xs font-medium uppercase tracking-wider text-neutral-500">
                  AA
                </th>
                <th className="px-6 py-3 text-center text-xs font-medium uppercase tracking-wider text-neutral-500">
                  Games Played
                </th>
                <th className="px-6 py-3 text-center text-xs font-medium uppercase tracking-wider text-neutral-500">
                  % Minutes Played
                </th>
                {["Goalkeeper", "Defender"].includes(usedParams.position) && (
                  <th
                    className={
                      "px-6 py-3 text-center text-xs font-medium uppercase tracking-wider text-neutral-500"
                    }
                  >
                    Avg Team GA
                  </th>
                )}
                {["Midfielder", "Forward"].includes(usedParams.position) && (
                  <th
                    className={
                      "px-6 py-3 text-center text-xs font-medium uppercase tracking-wider text-neutral-500"
                    }
                  >
                    Avg Team GF
                  </th>
                )}
                <th className="px-6 py-3 text-center text-xs font-medium uppercase tracking-wider text-neutral-500">
                  AA/Min
                </th>
                {usedParams.order.startsWith("duel_won_per_90_min") && (
                  <th className="px-6 py-3 text-center text-xs font-medium uppercase tracking-wider text-neutral-500">
                    Duel/90
                  </th>
                )}
                {usedParams.order.startsWith(
                  "effective_clearance_per_90_min"
                ) && (
                  <th className="px-6 py-3 text-center text-xs font-medium uppercase tracking-wider text-neutral-500">
                    EC/90
                  </th>
                )}
                {usedParams.order.startsWith("shot_on_target_per_90_min") && (
                  <th className="px-6 py-3 text-center text-xs font-medium uppercase tracking-wider text-neutral-500">
                    Shot/90
                  </th>
                )}
                {usedParams.order.startsWith("accurate_pass_per_90_min") && (
                  <th className="px-6 py-3 text-center text-xs font-medium uppercase tracking-wider text-neutral-500">
                    AP/90
                  </th>
                )}
                {usedParams.order.startsWith("won_contest_per_90_min") && (
                  <th className="px-6 py-3 text-center text-xs font-medium uppercase tracking-wider text-neutral-500">
                    WC/90
                  </th>
                )}
                <th className="px-6 py-3 text-center text-xs font-medium uppercase tracking-wider text-neutral-500">
                  Total Minutes
                </th>
                <th className="px-6 py-3 text-center text-xs font-medium uppercase tracking-wider text-neutral-500">
                  <div className="flex items-center justify-center gap-3">
                    <span>Score Percentages</span>
                    <div className="flex items-center justify-center gap-1">
                      <span className="rounded bg-red-600 p-1 text-2xs text-white">
                        {">80"}
                      </span>
                      <span className="rounded bg-orange-500 p-1 text-2xs text-white">
                        {">70"}
                      </span>
                      <span className="rounded bg-yellow-400 p-1 text-2xs text-black">
                        {">60"}
                      </span>
                      <span className="rounded bg-green-300 p-1 text-2xs text-black">
                        {">50"}
                      </span>
                    </div>
                  </div>
                </th>
                <th className="px-6 py-3 text-center text-xs font-medium uppercase tracking-wider text-gray-500">
                  <div className="flex items-center justify-center gap-3">
                    <span>AA Percentages</span>
                    <span className="rounded bg-red-600 p-1 text-2xs text-white">
                      {">40"}
                    </span>
                    <span className="rounded bg-orange-500 p-1 text-2xs text-white">
                      {">30"}
                    </span>
                    <span className="rounded bg-yellow-400 p-1 text-2xs text-black">
                      {">20"}
                    </span>
                    <span className="rounded bg-green-300 p-1 text-2xs text-black">
                      {">10"}
                    </span>
                  </div>
                </th>
              </tr>
            </thead>
            <tbody className="divide-y divide-neutral-200 bg-neutral-50">
              {data?.map((item) => {
                return (
                  <tr key={item.player.slug}>
                    <td className="relative whitespace-nowrap px-2 py-4">
                      <div
                        className={"flex h-10 w-10 items-center justify-center"}
                      >
                        <a
                          href={`https://www.soraredata.com/playerSearch/${item.player.displayName}`}
                          target="_blank"
                          rel="noopener noreferrer"
                        >
                          <img
                            src={item.player.avatarUrl}
                            className={"w-[115px]"}
                            alt={""}
                          />
                        </a>
                      </div>
                    </td>
                    <td className="relative whitespace-nowrap px-2 py-4">
                      <div
                        className={
                          "flex h-8 w-full items-center justify-center"
                        }
                      >
                        <Link to={`/team/${item.player.teamSlug}`}>
                          {item.player.team?.pictureUrl ? (
                            <img
                              src={item.player.team.pictureUrl}
                              className={"w-8"}
                              alt={""}
                              title={item.player.team?.shortName}
                            />
                          ) : (
                            <span
                              className="flex h-8 w-8 items-center justify-center rounded-full border-2 border-neutral-500 bg-white text-black"
                              title={item.player.team?.shortName}
                            >
                              {item.player.team?.shortName?.substring(0, 3)}
                            </span>
                          )}
                        </Link>
                      </div>
                    </td>
                    <td className="text-md px-2 py-4 font-secondary font-semibold">
                      <Link to={`/player/${item.player.slug}`}>
                        {item.player.displayName}
                      </Link>
                    </td>
                    <td className="whitespace-nowrap px-3 py-4 text-xs">
                      <div className={"flex items-baseline justify-center"}>
                        <span>
                          {calculateAge(new Date(item.player.birthDate))}
                        </span>
                      </div>
                    </td>
                    <td className="w-80 whitespace-nowrap px-2 py-2">
                      <div
                        className={
                          "flex flex-col items-center justify-center gap-3"
                        }
                      >
                        <div
                          className={"flex items-baseline justify-center gap-3"}
                        >
                          <span
                            className={
                              "w-8 text-center text-xs text-neutral-500"
                            }
                          >
                            {item.minScore}
                          </span>
                          <span
                            className={"w-10 text-center text-sm font-semibold"}
                          >
                            {item.totalAvgScore}
                          </span>
                          <span
                            className={
                              "w-8 text-center text-xs text-neutral-500"
                            }
                          >
                            {item.maxScore}
                          </span>
                        </div>
                        <div className="grid w-full grid-cols-5">
                          {item.scores
                            .slice(0, 5)
                            .reverse()
                            .map((score, index) => (
                              <span
                                key={index}
                                className={`text-xs text-white ${index > 0 ? "border-l border-neutral-400" : ""} w-full px-1 py-1 text-center 
                            ${score >= 80 ? "bg-green-600" : score >= 60 ? "bg-green-500" : score >= 40 ? "bg-yellow-400" : score >= 20 ? "bg-orange-500" : "bg-red-500"}
                            ${index === 0 ? "rounded-l-lg" : index === 4 ? "rounded-r-lg" : ""}`}
                              >
                                {Number(score) === -1
                                  ? "x"
                                  : Number(score).toFixed(0)}
                              </span>
                            ))}
                        </div>
                      </div>
                    </td>
                    <td className="whitespace-nowrap px-2 py-4 text-xs">
                      &plusmn;{item.totalStddevScore}
                    </td>
                    <td className="whitespace-nowrap px-2 py-4 text-sm">
                      {item.decisiveAvgScore}
                    </td>
                    <td className="whitespace-nowrap px-2 py-4 text-sm">
                      {item.allAroundAvgScore}
                    </td>
                    <td className="w-40 whitespace-nowrap px-2 py-4 text-sm">
                      <div className="flex w-full flex-col items-center gap-5">
                        <div className="w-full">
                          <ProgressBar
                            displayPercentage={false}
                            percentages={[
                              Math.round(
                                ((item.gameStartedCount +
                                  item.gameBenchedPlayedCount +
                                  item.gameBenchedUnplayedCount) /
                                  item.teamsGameCount) *
                                  100
                              ),
                              Math.round(
                                ((item.gameBenchedPlayedCount +
                                  item.gameBenchedUnplayedCount) /
                                  item.teamsGameCount) *
                                  100
                              ),
                              Math.round(
                                (item.gameBenchedUnplayedCount /
                                  item.teamsGameCount) *
                                  100
                              ),
                            ]}
                          />
                        </div>
                        <div className="flex w-full justify-between gap-3">
                          <span
                            className="rounded-md bg-orange-500 px-2 py-1 text-xs text-white"
                            title="Games Benched Unplayed"
                          >
                            {item.gameBenchedUnplayedCount}
                          </span>
                          <span
                            className="rounded-md bg-yellow-500 px-2 py-1 text-xs text-white"
                            title="Games Benched Played"
                          >
                            {item.gameBenchedPlayedCount}
                          </span>
                          <span
                            className="rounded-md bg-green-300 px-2 py-1 text-xs text-black"
                            title="Games Started"
                          >
                            {item.gameStartedCount}
                          </span>
                          <span
                            className="rounded-md bg-neutral-100 px-2 py-1 text-xs text-black"
                            title="Games Started"
                          >
                            {item.teamsGameCount}
                          </span>
                        </div>
                      </div>
                    </td>
                    <td className="whitespace-nowrap px-2 py-4 text-center text-sm">
                      {item.percentageMinutesPlayed}
                    </td>
                    {["Goalkeeper", "Defender"].includes(
                      usedParams.position
                    ) && (
                      <td
                        className="whitespace-nowrap px-2 py-4 text-sm"
                        text-center
                      >
                        {item.avgTeamGoalsAgainst}
                      </td>
                    )}
                    {["Midfielder", "Forward"].includes(
                      usedParams.position
                    ) && (
                      <td className="whitespace-nowrap px-2 py-4 text-center text-sm ">
                        {item.avgTeamGoalsFor}
                      </td>
                    )}
                    <td className="whitespace-nowrap px-2 py-4 text-center text-sm ">
                      {item.aaPerMin}
                    </td>
                    {usedParams.order.startsWith("duel_won_per_90_min") && (
                      <td className="whitespace-nowrap px-2 py-4 text-center text-sm ">
                        {item.duelWonPer90Min}
                      </td>
                    )}
                    {usedParams.order.startsWith(
                      "effective_clearance_per_90_min"
                    ) && (
                      <td className="whitespace-nowrap px-2 py-4 text-center text-sm ">
                        {item.effectiveClearancePer90Min}
                      </td>
                    )}
                    {usedParams.order.startsWith(
                      "shot_on_target_per_90_min"
                    ) && (
                      <td className="whitespace-nowrap px-2 py-4 text-center text-sm ">
                        {item.shotOnTargetPer90Min}
                      </td>
                    )}
                    {usedParams.order.startsWith(
                      "accurate_pass_per_90_min"
                    ) && (
                      <td className="whitespace-nowrap px-2 py-4 text-center text-sm ">
                        {item.accuratePassPer90Min}
                      </td>
                    )}
                    {usedParams.order.startsWith("won_contest_per_90_min") && (
                      <td className="whitespace-nowrap px-2 py-4 text-center text-sm ">
                        {item.wonContestPer90Min}
                      </td>
                    )}
                    <td className="whitespace-nowrap px-2 py-4 text-center text-sm ">
                      {item.totalMinutes}
                    </td>
                    <td className="whitespace-nowrap px-2 py-4">
                      <ProgressBar
                        percentages={[
                          item.moreThan80ScorePercentage,
                          item.moreThan70ScorePercentage,
                          item.moreThan60ScorePercentage,
                          item.moreThan50ScorePercentage,
                        ]}
                      />
                    </td>
                    <td className="whitespace-nowrap px-2 py-4">
                      <ProgressBar
                        percentages={[
                          item.moreThan40AAPercentage,
                          item.moreThan30AAPercentage,
                          item.moreThan20AAPercentage,
                          item.moreThan10AAPercentage,
                        ]}
                      />
                    </td>
                  </tr>
                )
              })}
            </tbody>
          </table>
        </div>
      )}
    </>
  )
}

export default SingleRankingsPage
