import { useMemo } from "react"
import { Link, useParams } from "react-router-dom"
import { GetTeamRoster } from "../../api/team"
import man from "../../assets/man.png"
import WithDataFetching from "../../components/withdatafetching"
import { GamePlayer } from "../../types/game"
import Player from "../../types/player"
import Team from "../../types/team"

export default function Roster() {
  const { slug } = useParams()
  if (!slug) {
    return <div>No</div>
  }
  return (
    <WithDataFetching
      queryKey={["roster", slug]}
      queryFn={() => GetTeamRoster(slug)}
    >
      {(resp) => <RosterLayout team={resp.team} roster={resp.roster} />}
    </WithDataFetching>
  )
}

function RosterLayout({ team, roster }: { team: Team; roster: Player[] }) {
  const threeMonthsAgo = new Date()
  threeMonthsAgo.setMonth(threeMonthsAgo.getMonth() - 3)
  const totalTeamMinutes = useMemo(() => {
    return team.homeGames?.concat(team.awayGames)?.reduce((total, game) => {
      if (new Date(game.date) >= threeMonthsAgo) {
        return total + (game.minutes > 115 ? 120 : 90)
      }
      return total
    }, 0)
  }, [team])

  const computeTotalMinutes = (gamePlayers: GamePlayer[]) => {
    return gamePlayers.reduce((total, game) => {
      if (game.game && new Date(game.game.date) >= threeMonthsAgo) {
        return total + (game.score?.minutesPlayed || 0)
      }
      return total
    }, 0)
  }

  const rosterWithMinutes = roster.map((player) => ({
    ...player,
    totalMinutesPlayed: computeTotalMinutes(player.gamePlayers),
    minutesPlayedPercentage:
      (computeTotalMinutes(player.gamePlayers) / totalTeamMinutes) * 100,
  }))

  return (
    <div className="flex flex-col gap-3">
      {["Goalkeeper", "Defender", "Midfielder", "Forward"].map((position) => (
        <>
          <div className="rounded-xl bg-gradient-to-br from-primary-200 to-primary-50 px-3 py-1 text-center text-xl font-semibold text-neutral-700 shadow-md">
            {position}s
          </div>
          <div className="grid grid-cols-10 gap-3 p-3">
            {rosterWithMinutes
              .filter((player) => player.fieldPosition === position)
              .sort((a, b) => b.totalMinutesPlayed - a.totalMinutesPlayed)
              .map((player) => (
                <div
                  className="flex flex-col items-center gap-2"
                  key={player.slug}
                >
                  <Link to={`/player/${player.slug}`}>
                    <img
                      className="h-32 rounded-md"
                      src={player.avatarUrl || man}
                      alt={""}
                    />
                  </Link>
                  <div>{player.displayName}</div>
                  <div>{player.totalMinutesPlayed}</div>
                  <div>{player.minutesPlayedPercentage.toFixed(1)}%</div>
                </div>
              ))}
          </div>
        </>
      ))}
    </div>
  )
}
