import React from "react"
import { FaPersonWalkingArrowRight } from "react-icons/fa6"
import { Link, useParams } from "react-router-dom"
import { GetTeamMemberships } from "../../api/team"
import man from "../../assets/man.png"
import Badge from "../../components/badge"
import WithDataFetching from "../../components/withdatafetching"
import { Membership } from "../../types/player"
import Team from "../../types/team"

export default function Transfers() {
  const { slug } = useParams()
  if (!slug) {
    return <div>No team slug</div>
  }

  return (
    <WithDataFetching
      queryKey={["memberships", slug]}
      queryFn={() => GetTeamMemberships(slug)}
    >
      {(resp) => (
        <TransfersLayout team={resp.team} memberships={resp.memberships} />
      )}
    </WithDataFetching>
  )
}

function TransfersLayout({
  memberships,
}: {
  team: Team
  memberships: Membership[]
}) {
  type YearMonthAccumulator = {
    [key: string]: {
      arrivals: Membership[]
      departures: Membership[]
    }
  }
  const groupedByYearMonth = memberships.reduce<YearMonthAccumulator>(
    (acc, membership) => {
      const startYearMonth = new Date(membership.startDate)
        .toISOString()
        .slice(0, 7)
      const endYearMonth = membership.endDate
        ? new Date(membership.endDate).toISOString().slice(0, 7)
        : null

      if (!acc[startYearMonth]) {
        acc[startYearMonth] = { arrivals: [], departures: [] }
      }
      acc[startYearMonth].arrivals.push(membership)

      if (endYearMonth) {
        if (!acc[endYearMonth]) {
          acc[endYearMonth] = { arrivals: [], departures: [] }
        }
        acc[endYearMonth].departures.push(membership)
      }

      return acc
    },
    {}
  )

  // Sort each group by startDate and then by playerSlug
  Object.keys(groupedByYearMonth).forEach((yearMonth) => {
    groupedByYearMonth[yearMonth].arrivals.sort((a, b) => {
      if (a.startDate === b.startDate) {
        return a.playerSlug.localeCompare(b.playerSlug)
      }
      return a.startDate < b.startDate ? -1 : 1
    })
    groupedByYearMonth[yearMonth].departures.sort((a, b) => {
      if (a.startDate === b.startDate) {
        return a.playerSlug.localeCompare(b.playerSlug)
      }
      return a.startDate < b.startDate ? -1 : 1
    })
  })

  const sortedYearMonths = Object.keys(groupedByYearMonth).sort().reverse()

  return (
    <div>
      {sortedYearMonths.map((yearMonth, index, array) => {
        const { arrivals, departures } = groupedByYearMonth[yearMonth]
        const year = yearMonth.slice(0, 4)
        const isYearBoundary =
          index === 0 || year !== array[index - 1].slice(0, 4)
        const isEvenMonth = index % 2 === 0

        return (
          <React.Fragment key={yearMonth}>
            {isYearBoundary && (
              <div className="flex w-full items-center justify-center rounded-xl bg-gradient-to-br from-primary-200 to-primary-50 px-3 py-1 text-xl font-semibold text-neutral-700 shadow-md">
                {year}
              </div>
            )}
            <div
              className={`m-3
                grid grid-cols-[1fr_240px_1fr] items-center rounded-lg p-3 ${isEvenMonth ? "bg-neutral-200" : "bg-neutral-50"}`}
            >
              <div className="grid grid-cols-2 gap-3">
                {arrivals.map((m) => {
                  const previousMembership = m.player?.clubMembership
                    ?.filter(
                      (membership) => membership.membershipType === "club"
                    )
                    .sort((a, b) => {
                      if (!a.endDate && !b.endDate) {
                        return (
                          new Date(b.startDate).getTime() -
                          new Date(a.startDate).getTime()
                        )
                      }
                      if (!a.endDate) return -1
                      if (!b.endDate) return 1
                      if (
                        new Date(a.endDate).getTime() ===
                        new Date(b.endDate).getTime()
                      ) {
                        return (
                          new Date(b.startDate).getTime() -
                          new Date(a.startDate).getTime()
                        )
                      }
                      return (
                        new Date(b.endDate).getTime() -
                        new Date(a.endDate).getTime()
                      )
                    })
                    .find(
                      (membership) =>
                        new Date(membership.startDate) < new Date(m.startDate)
                    )
                  return (
                    <div key={m.id} className="flex items-center gap-3">
                      <Link to={`/player/${m.player?.slug}`}>
                        <img
                          className="h-24 min-w-16"
                          src={m.player?.avatarUrl || man}
                          alt={m.player?.displayName}
                        />
                      </Link>
                      <div className="flex flex-col items-start gap-1">
                        <span className="font-secondary text-lg font-semibold">
                          {m.player?.displayName}
                        </span>
                        <span className="flex items-center gap-3">
                          {previousMembership && previousMembership.team ? (
                            <>
                              <FaPersonWalkingArrowRight className="h-6 w-6" />
                              <Badge
                                name={previousMembership?.team?.shortName}
                                imgUrl={previousMembership?.team?.pictureUrl}
                                link={`/team/${previousMembership?.team?.slug}`}
                              />
                            </>
                          ) : (
                            "No previous club"
                          )}
                        </span>
                      </div>
                    </div>
                  )
                })}
              </div>
              <div className="rounded-xl px-20 text-center text-xl text-neutral-700">
                <span className="rounded-xl border border-neutral-700 bg-neutral-500 px-2 py-1 text-white shadow-md">
                  {new Date(yearMonth + "-01")
                    .toLocaleString(undefined, { month: "long" })
                    .replace(/^\w/, (c) => c.toUpperCase())}
                </span>
              </div>
              <div className="grid grid-cols-2 gap-3">
                {departures.map((m) => {
                  const nextMembership = m.player?.clubMembership
                    ?.filter(
                      (membership) => membership.membershipType === "club"
                    )
                    .sort((a, b) => {
                      if (!a.endDate && !b.endDate) {
                        return (
                          new Date(b.startDate).getTime() -
                          new Date(a.startDate).getTime()
                        )
                      }
                      if (!a.endDate) return -1
                      if (!b.endDate) return 1
                      if (
                        new Date(a.endDate).getTime() ===
                        new Date(b.endDate).getTime()
                      ) {
                        return (
                          new Date(b.startDate).getTime() -
                          new Date(a.startDate).getTime()
                        )
                      }
                      return (
                        new Date(b.endDate).getTime() -
                        new Date(a.endDate).getTime()
                      )
                    })
                    .reverse()
                    .find(
                      (membership) =>
                        new Date(membership.startDate) >= new Date(m.endDate)
                    )
                  return (
                    <div key={m.id} className="flex items-center gap-3">
                      <Link to={`/player/${m.player?.slug}`}>
                        <img
                          className="h-24 min-w-16"
                          src={m.player?.avatarUrl || man}
                          alt={m.player?.displayName}
                        />
                      </Link>
                      <div className="flex flex-col items-start gap-1">
                        <span className="font-secondary text-lg font-semibold">
                          {m.player?.displayName}
                        </span>
                        <span className="flex items-center gap-3">
                          {nextMembership && nextMembership.team ? (
                            <>
                              <Badge
                                name={nextMembership?.team?.shortName}
                                imgUrl={nextMembership?.team?.pictureUrl}
                                link={`/team/${nextMembership?.team?.slug}`}
                              />
                              <FaPersonWalkingArrowRight className="h-6 w-6" />
                            </>
                          ) : (
                            "No next club"
                          )}
                        </span>
                      </div>
                    </div>
                  )
                })}
              </div>
            </div>
          </React.Fragment>
        )
      })}
    </div>
  )
}
