import { useEffect, useMemo, useState } from "react"
import { FaArrowAltCircleUp } from "react-icons/fa"
import { IoMdFootball } from "react-icons/io"
import { IoManSharp } from "react-icons/io5"
import { useParams } from "react-router"
import { Link } from "react-router-dom"
import Select from "react-select"
import { GetUserCards } from "../../api/card"
import WithDataFetching from "../../components/withdatafetching"
import Card from "../../types/card"

export default function XpCenter() {
  const { slug = "gigiz22" } = useParams()

  return (
    <WithDataFetching<Card[]>
      queryKey={["user", slug, "cards"]}
      queryFn={() => GetUserCards(slug)}
      refetchOnWindowFocus={false}
    >
      {(data) => <CardList cards={data} />}
    </WithDataFetching>
  )
}

function CardList({ cards }: { cards: Card[] }) {
  const [sortConfig, setSortConfig] = useState<{
    key: string
    direction: "asc" | "desc"
  }>(
    () =>
      JSON.parse(
        localStorage.getItem("sortConfig") ||
          '{"key": "xpPercentage", "direction": "desc"}'
      ) as {
        key: string
        direction: "asc" | "desc"
      }
  )

  const [hideMaxLevelUp, setHideMaxLevelUp] = useState(() =>
    JSON.parse(localStorage.getItem("hideMaxLevelUp") || "true")
  )
  const [hideMaxLevelCards, setHideMaxLevelCards] = useState(() =>
    JSON.parse(localStorage.getItem("hideMaxLevelCards") || "true")
  )
  const [minGrade, setMinGrade] = useState<number>(() =>
    JSON.parse(localStorage.getItem("minGrade") || "0")
  )
  const [onlyInSeason, setOnlyInSeason] = useState(() =>
    JSON.parse(localStorage.getItem("onlyInSeason") || "false")
  )

  useEffect(() => {
    localStorage.setItem("sortConfig", JSON.stringify(sortConfig))
  }, [sortConfig])

  useEffect(() => {
    localStorage.setItem("hideMaxLevelUp", JSON.stringify(hideMaxLevelUp))
  }, [hideMaxLevelUp])

  useEffect(() => {
    localStorage.setItem("hideMaxLevelCards", JSON.stringify(hideMaxLevelCards))
  }, [hideMaxLevelCards])

  useEffect(() => {
    localStorage.setItem("minGrade", JSON.stringify(minGrade))
  }, [minGrade])

  useEffect(() => {
    localStorage.setItem("onlyInSeason", JSON.stringify(onlyInSeason))
  }, [onlyInSeason])

  const filteredCards = useMemo(() => {
    return cards.filter((card) => {
      if (
        hideMaxLevelUp &&
        card.levelUpAppliedCount >= card.maxLevelUpAppliedCount
      ) {
        return false
      }
      if (hideMaxLevelCards && card.grade >= 20) {
        return false
      }
      if (card.grade < minGrade) {
        return false
      }
      if (onlyInSeason && !card.inSeasonEligible) {
        return false
      }
      return true
    })
  }, [cards, hideMaxLevelUp, hideMaxLevelCards, minGrade, onlyInSeason])

  const sortedCards = useMemo(() => {
    let sortableCards = filteredCards.map((card) => ({
      ...card,
      xpNeeded:
        card.xpNeededForNextGrade > card.xp
          ? card.xpNeededForNextGrade - card.xp
          : "N/A",
      xpPercentage:
        card.xpNeededForNextGrade > card.xp
          ? ((card.xpNeededForNextGrade - card.xp) * 100) /
            (card.xpNeededForNextGrade - card.xpNeededForCurrentGrade)
          : "N/A",
    }))

    if (sortConfig !== null) {
      sortableCards.sort((a, b) => {
        const key = sortConfig.key as keyof typeof a
        if (a[key] === "N/A") return 1
        if (b[key] === "N/A") return -1
        if (Number(a[key]) < Number(b[key])) {
          return sortConfig.direction === "asc" ? -1 : 1
        }
        if (Number(a[key]) > Number(b[key])) {
          return sortConfig.direction === "asc" ? 1 : -1
        }
        return 0
      })
    }
    return sortableCards
  }, [filteredCards, sortConfig])

  const handleSortFieldChange = (selectedOption: any) => {
    setSortConfig((prevConfig) => ({
      key: selectedOption.value,
      direction: prevConfig?.direction || "asc",
    }))
  }

  const handleSortDirectionChange = (selectedOption: any) => {
    setSortConfig((prevConfig) => ({
      key: prevConfig?.key || "",
      direction: selectedOption.value,
    }))
  }

  const sortFieldOptions = [
    { value: "grade", label: "Grade" },
    { value: "xp", label: "XP" },
    { value: "xpNeeded", label: "XP Needed for Next Grade" },
    { value: "xpPercentage", label: "% Until next grade" },
  ]

  const sortDirectionOptions = [
    { value: "asc", label: "Ascending" },
    { value: "desc", label: "Descending" },
  ]

  return (
    <div className="flex flex-col gap-3 p-3">
      <div className="flex w-full items-center justify-center gap-3">
        <label className="inline-flex items-center">
          <span className="mr-2">Min Grade</span>
          <input
            type="number"
            value={minGrade}
            onChange={(e) => setMinGrade(Number(e.target.value))}
            className="w-16"
          />
        </label>

        <label className="inline-flex items-center">
          <input
            type="checkbox"
            checked={onlyInSeason}
            onChange={() => setOnlyInSeason(!onlyInSeason)}
          />
          <span className="ml-2">Only show cards in season</span>
        </label>

        <label className="inline-flex items-center">
          <input
            type="checkbox"
            checked={hideMaxLevelUp}
            onChange={() => setHideMaxLevelUp(!hideMaxLevelUp)}
          />
          <span className="ml-2">Hide cards with max level up applied</span>
        </label>

        <label className="inline-flex items-center">
          <input
            type="checkbox"
            checked={hideMaxLevelCards}
            onChange={() => setHideMaxLevelCards(!hideMaxLevelCards)}
          />
          <span className="ml-2">Hide cards with max grade</span>
        </label>

        <Select
          options={sortFieldOptions}
          onChange={handleSortFieldChange}
          defaultValue={sortFieldOptions.find(
            (option) => option.value === sortConfig.key
          )}
          placeholder="Sort by"
          className="w-64"
        />
        <Select
          options={sortDirectionOptions}
          onChange={handleSortDirectionChange}
          defaultValue={sortDirectionOptions.find(
            (option) => option.value === sortConfig.direction
          )}
          placeholder="Sort direction"
        />
      </div>
      <div className="grid grid-cols-10 gap-3">
        {sortedCards.map((card) => (
          <div
            key={card.id}
            className="flex flex-col gap-3 rounded-xl border-2 border-neutral-300 pb-3 pt-0.5 shadow-lg"
          >
            <Link to={`/card/${card.assetId}`}>
              <img src={card.pictureUrl} alt={card.name} className=" px-0.5" />
            </Link>

            <div className="flex flex-col gap-3 px-3">
              <div className="flex items-center justify-center gap-3">
                <Link to={`/player/${card.playerSlug}`}>
                  <IoManSharp className="text-lg" />
                </Link>
                <a
                  href={`https://sorare.com/fr/football/cards/${card.slug}`}
                  target="_blank"
                  rel="noopener noreferrer"
                >
                  <IoMdFootball className="text-lg" title="Sorare" />
                </a>
              </div>
              <div className="flex items-baseline justify-between">
                <span className="rounded-full bg-gradient-to-br from-neutral-700 to-neutral-500 px-2 py-1 text-xs font-semibold text-white shadow-lg">
                  {card.grade}
                </span>
                <span className="text-xs font-semibold text-neutral-700">
                  {card.grade < 20
                    ? `${card.xp - card.xpNeededForCurrentGrade} / ${card.xpNeededForNextGrade - card.xpNeededForCurrentGrade}`
                    : card.xp}
                </span>
                <span className="text-xs font-semibold text-neutral-700">
                  {Number((card.power - 1) * 100).toFixed(1)}%
                </span>
              </div>
              <div className="flex items-center gap-1">
                <div className="flex h-3 w-full gap-1 overflow-hidden rounded bg-neutral-300 text-xs">
                  <div
                    style={{
                      width: `${((card.xp - card.xpNeededForCurrentGrade) / (card.xpNeededForNextGrade - card.xpNeededForCurrentGrade)) * 100}%`,
                    }}
                    className="flex flex-col justify-center whitespace-nowrap bg-primary-700 text-center text-white shadow-none"
                  ></div>
                </div>
                <span className="text-2xs font-semibold text-neutral-700">
                  {card.grade < 20
                    ? `${Number(card.xpPercentage).toFixed(1)}%`
                    : ""}
                </span>
              </div>
            </div>
            <div className="flex items-center justify-center gap-3 px-3">
              <FaArrowAltCircleUp
                className={
                  card.levelUpAppliedCount < 1
                    ? "text-neutral-300"
                    : "text-green-600"
                }
              />
              <FaArrowAltCircleUp
                className={
                  card.levelUpAppliedCount < 2
                    ? "text-neutral-300"
                    : "text-green-600"
                }
              />
              <FaArrowAltCircleUp
                className={
                  card.levelUpAppliedCount < 3
                    ? "text-neutral-300"
                    : "text-green-600"
                }
              />
            </div>
            <div className="flex items-center justify-center px-3">
              {card.inSeasonEligible ? (
                <span className="rounded-xl bg-primary-600 px-2 py-1 text-xs font-semibold text-white">
                  In Season
                </span>
              ) : (
                <span>&nbsp;</span>
              )}
            </div>
          </div>
        ))}
      </div>
    </div>
  )
}
